package org.appfuse.web;

import java.util.List;

import net.sourceforge.jwebunit.WebTestCase;

import org.appfuse.dao.UserDAO;
import org.appfuse.model.User;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

public class UserWebTest extends WebTestCase {
    
    public UserWebTest(String name) {
        super(name);
        getTestContext().setBaseUrl("http://localhost:8080/myusers");
    }
    
    public void testWelcomePage() {
        beginAt("/");
        assertTitleEquals("MyUsers ~ Welcome");
    }
    
    public void testAddUser() {
        beginAt("/editUser.html");
        assertTitleEquals("MyUsers ~ User Details");
        setFormElement("firstName", "Spring");
        setFormElement("lastName", "User");
        submit("save");
        assertTextPresent("saved successfully");
    }
    
    public void testListUsers() {
        beginAt("/users.html");     
        
        // check that table is present
        assertTablePresent("userList");

        //check that a set of strings are present somewhere in table
        assertTextInTable("userList",
                          new String[] {"Spring", "User"});
    }
    
    public void testEditUser() {
        beginAt("/editUser.html?id=" + getInsertedUserId());
        assertFormElementEquals("firstName", "Spring");
        submit("save");
        assertTitleEquals("MyUsers ~ User List");
    }
    
    public void testDeleteUser() {
        beginAt("/editUser.html?id=" + getInsertedUserId());
        assertTitleEquals("MyUsers ~ User Details");
        submit("delete");
        assertTitleEquals("MyUsers ~ User List");
    }
    
    /**
     * Convenience method to get the id of the inserted user
     * Assumes last inserted user is "Spring User"
     */
    public String getInsertedUserId() {
        String[] paths = {"/WEB-INF/applicationContext.xml"};
        ApplicationContext ctx = 
            new ClassPathXmlApplicationContext(paths);
        List users = ((UserDAO) ctx.getBean("userDAO")).getUsers();
        // assumed that user inserted in testAddUser() is last user
        return ""+((User)users.get(users.size()-1)).getId();
    }
}